# Core Library modules
from pathlib import WindowsPath

# Third party modules
import pytest

# First party modules
from pynball import pynball

BASE = WindowsPath(__file__).resolve().parents[3]
VERSIONS = ["3.6.8", "3.7.9", "3.9.10"]


def test_get_pynball_notype(capsys, mock_get_env) -> None:
    pynball._get_pynball("nosuchthing", "PYNBALL")
    captured = capsys.readouterr()
    assert (
        captured.out == "WARNING: Please use a correct returntype - "
        "('string', 'dict', 'dict_path_object', 'names', 'paths')\n"
    )


def test_get_pynball_string(mock_get_env) -> None:
    """generated string should look similar to the following
    pynball._get_pynball("string", "PYNBALL") == (
        "{'3.6.8': 'D:\\\\PYTHON "
        "PROJECT\\\\PROJECTS\\\\DEV\\\\pynball\\\\pynball\\\\env\\\\PYTHON\\\\3.6.8', "
        "'3.7.9': 'D:\\\\PYTHON "
        "PROJECT\\\\PROJECTS\\\\DEV\\\\pynball\\\\pynball\\\\env\\\\PYTHON\\\\3.7.9', "
        "'3.9.10': 'D:\\\\PYTHON "
        "PROJECT\\\\PROJECTS\\\\DEV\\\\pynball\\\\pynball\\\\env\\\\PYTHON\\\\3.9.10'}"
    )
    """
    get_str = pynball._get_pynball("string", "PYNBALL")
    for ver in VERSIONS:
        assert f"pynball\\\\pynball\\\\env\\\\PYTHON\\\\{ver}" in get_str


def test_get_pynball_dict(mock_get_env) -> None:
    """Generated dictionary should look similar to the following
    pynball._get_pynball("dict", "PYNBALL") == {
        "3.6.8": "D:\\PYTHON "
        "PROJECT\\PROJECTS\\DEV\\pynball\\pynball\\env\\PYTHON\\3.6.8",
        "3.7.9": "D:\\PYTHON "
        "PROJECT\\PROJECTS\\DEV\\pynball\\pynball\\env\\PYTHON\\3.7.9",
        "3.9.10": "D:\\PYTHON "
        "PROJECT\\PROJECTS\\DEV\\pynball\\pynball\\env\\PYTHON\\3.9.10",
    }
    """
    get_dict = pynball._get_pynball("dict", "PYNBALL")
    for ver in VERSIONS:
        assert f"pynball\\pynball\\env\\PYTHON\\{ver}" in get_dict[f"{ver}"]


def test_get_pynball_object(mock_get_env) -> None:
    """Generated dictionary should look similar to the following:
    pynball._get_pynball("dict_path_object", "PYNBALL") == {
        "3.6.8": WindowsPath(
            "D:/PYTHON PROJECT/PROJECTS/DEV/pynball/pynball/env/PYTHON/3.6.8"
        ),
        "3.7.9": WindowsPath(
            "D:/PYTHON PROJECT/PROJECTS/DEV/pynball/pynball/env/PYTHON/3.7.9"
        ),
        "3.9.10": WindowsPath(
            "D:/PYTHON PROJECT/PROJECTS/DEV/pynball/pynball/env/PYTHON/3.9.10"
        ),
    }
    """
    path_object_dict = pynball._get_pynball("dict_path_object", "PYNBALL")
    for ver in VERSIONS:
        assert str(WindowsPath(f"pynball/pynball/env/PYTHON/{ver}")) in str(
            path_object_dict[f"{ver}"]
        )


def test_get_pynball_names(mock_get_env) -> None:
    assert pynball._get_pynball("names", "PYNBALL") == ["3.6.8", "3.7.9", "3.9.10"]


def test_get_pynball_paths(mock_get_env) -> None:
    """Generated list should look similar to the following
    assert pynball._get_pynball("paths", "PYNBALL") == [
        "D:\\PYTHON PROJECT\\PROJECTS\\DEV\\pynball\\pynball\\env\\PYTHON\\3.6.8",
        "D:\\PYTHON PROJECT\\PROJECTS\\DEV\\pynball\\pynball\\env\\PYTHON\\3.7.9",
        "D:\\PYTHON PROJECT\\PROJECTS\\DEV\\pynball\\pynball\\env\\PYTHON\\3.9.10",
    ]
    """
    paths_list = pynball._get_pynball("paths", "PYNBALL")
    for index, ver in enumerate(VERSIONS):
        assert f"pynball\\pynball\\env\\PYTHON\\{ver}" in paths_list[index]



# Core Library modules
from pathlib import Path

# Third party modules
import pytest

# First party modules
from pynball import pynball


def test_set_pynball1(capsys, mock_set_env_output):
    dict_object = {"3.6": Path("D:\\Python\\python3.6")}
    pynball._set_pynball(dict_object, "PYNBALL")
    output, error = capsys.readouterr()
    assert output == "{'3.6': 'D:\\\\Python\\\\python3.6'}\n"
    assert error == ""



# Core Library modules
from pathlib import Path, PurePath, WindowsPath

# Third party modules
import pytest
from click.testing import CliRunner

# First party modules
from pynball import pynball


def test_add(create_env, mock_get_env, mock_set_env):
    runner = CliRunner()
    new_path = str(pytest.PYTHON_DIR / "3.8.10")
    result = runner.invoke(pynball.add, ["3.8.10", new_path])
    assert result.exit_code == 0
    assert result.output == "'3.8.10' Successfully added to configuration\n"


def test_add_noexe(create_env, mock_get_env, mock_set_env):
    runner = CliRunner()
    new_path = str(pytest.PYTHON_DIR / "3.6.8")
    result = runner.invoke(pynball.add, ["3.6.8", new_path])
    assert result.exit_code == 0
    assert result.output == "WARNING: There is no Python Interpreter on that path\n"


def test_add_exists(create_env, mock_get_env, mock_set_env):
    runner = CliRunner()
    new_path = str(pytest.PYTHON_DIR / "3.7.9")
    result = runner.invoke(pynball.add, ["3.7.9", new_path])
    assert result.exit_code == 0
    assert (
        result.output == "WARNING: '3.7.9' already added to configuration as '3.7.9'\n"
    )



# Core Library modules
from pathlib import Path, PurePath, WindowsPath

# Third party modules
import pytest
from click.testing import CliRunner

# First party modules
from pynball import pynball


def test_lsproject(monkeypatch, create_env, mock_get_env):
    runner = CliRunner()
    monkeypatch.setattr(pynball, "_WORKON_HOME", pytest.WORKON_HOME)
    monkeypatch.setattr(pynball, "_PROJECT_HOME", pytest.PROJECT_HOME)

    result = runner.invoke(pynball.lsproject)
    assert result.exit_code == 0
    assert result.output == (
        "Project Name             System Version           Pyenv Versions\n"
        "============             ==============           ==============\n"
        "project1                 3.9.10                   3.8.10, 3.9.10\n"
        "project2                 3.9.10                   3.8.10, 3.9.10\n"
    )



# Core Library modules
from pathlib import Path, PurePath, WindowsPath

# Third party modules
import pytest
from click.testing import CliRunner

# First party modules
from pynball import pynball

VERSIONS = ["3.6.8", "3.7.9", "3.9.10"]


def test_versions(mock_get_env) -> None:
    runner = CliRunner()
    result = runner.invoke(pynball.versions)
    assert result.exit_code == 0
    """
    assert result.output == (
        "WARNING: System Interpreter is not configured\n"
        "3.6.8     D:\\PYTHON "
        "PROJECT\\PROJECTS\\DEV\\pynball\\pynball\\env\\PYTHON\\3.6.8\n"
        "3.7.9     D:\\PYTHON "
        "PROJECT\\PROJECTS\\DEV\\pynball\\pynball\\env\\PYTHON\\3.7.9\n"
        "3.9.10    D:\\PYTHON "
        "PROJECT\\PROJECTS\\DEV\\pynball\\pynball\\env\\PYTHON\\3.9.10\n"
    )
    """

    assert "WARNING: System Interpreter is not configured" in result.output
    for ver in VERSIONS:
        assert f"pynball\\pynball\\env\\PYTHON\\{ver}" in result.output



# Third party modules
import pytest

# First party modules
from pynball import pynball


@pytest.mark.dependency()
def test_set_env(capsys):
    """Test case to check environmental variable getter and setter methods"""
    # Setter
    pynball._setenv("unknown", "DELETEKEY", "deletevalue")
    output, error = capsys.readouterr()
    assert output == "WARNING: Scope value must be 'user' or 'system'\n"
    assert error == ""
    pynball._setenv("user", "DELETEKEY", "deletevalue")
    output, error = capsys.readouterr()
    assert output == ""
    assert error == ""
    pynball._setenv("system", "DELETEKEY", "deletevalue")
    output, error = capsys.readouterr()
    assert output == ""
    assert error == ""


@pytest.mark.dependency(depends=["test_set_env"])
def test_get_env(capsys):
    pynball._getenv("unknown", "DELETEKEY")
    output, error = capsys.readouterr()
    assert output == "WARNING: Scope value must be 'user' or 'system'\n"
    assert error == ""
    pynball._getenv("user", "DELETEKEY")
    output, error = capsys.readouterr()
    assert output == ""
    assert error == ""
    pynball._getenv("system", "DELETEKEY")
    output, error = capsys.readouterr()
    assert output == ""
    assert error == ""


@pytest.mark.dependency(depends=["test_set_env"])
def test_del_env(capsys):
    pynball._delenv("unknown", "DELETEKEY")
    output, error = capsys.readouterr()
    assert output == "WARNING: Scope value must be 'user' or 'system'\n"
    assert error == ""
    pynball._delenv("user", "DELETEKEY")
    output, error = capsys.readouterr()
    assert output == ""
    assert error == ""
    pynball._delenv("system", "DELETEKEY")
    output, error = capsys.readouterr()
    assert output == ""
    assert error == ""



# Core Library modules
from pathlib import Path, PurePath, WindowsPath

# Third party modules
import pytest
from click.testing import CliRunner

# First party modules
from pynball import pynball


def test_rmproject_nove(monkeypatch, create_env, mock_get_env):
    runner = CliRunner()
    monkeypatch.setattr(pynball, "_WORKON_HOME", Path(""))
    monkeypatch.setattr(pynball, "_PROJECT_HOME", pytest.PROJECT_HOME)

    result = runner.invoke(pynball.rmproject, ["project2"])
    assert result.exit_code == 0
    assert result.output == (
        "WARNING: Virtualenv-wrapper is not configured on your system:\n"
        "        Please install Virtualenv and Virtualenv-wrapper and configure\n"
        "        'WORKON_HOME' and 'PROJECT_HOME' environment variables\n"
    )


def test_rmproject_noproject(monkeypatch, create_env, mock_get_env):
    runner = CliRunner()
    monkeypatch.setattr(pynball, "_WORKON_HOME", pytest.WORKON_HOME)
    monkeypatch.setattr(pynball, "_PROJECT_HOME", pytest.PROJECT_HOME)

    result = runner.invoke(pynball.rmproject, ["project3"])
    assert result.output == "WARNING: Project: 'project3' does not exist\n"


def test_rmproject_delvenv(monkeypatch, create_env, mock_get_env):
    """test deletion of venv directory"""
    pass


def test_rmproject_deldev(monkeypatch, create_env, mock_get_env):
    """test deletion of venv and dev directories"""
    pass



# Core Library modules
from pathlib import Path, PurePath, WindowsPath

# Third party modules
import pytest
from click.testing import CliRunner

# First party modules
from pynball import pynball


def test_delete(create_env, mock_get_env, mock_set_env):
    runner = CliRunner()
    result = runner.invoke(pynball.delete, ["3.6.8"])
    assert result.exit_code == 0
    assert result.output == ""


def test_delete_noentry(create_env, mock_get_env, mock_set_env):
    runner = CliRunner()
    result = runner.invoke(pynball.delete, ["3.5.3"])
    assert result.exit_code == 0
    assert result.output == ""



# Core Library modules
from pathlib import Path, PurePath, WindowsPath

# Third party modules
import pytest
from click.testing import CliRunner

# First party modules
from pynball import pynball


def test_version(mock_get_env):
    runner = CliRunner()
    result = runner.invoke(pynball.version)
    assert result.exit_code == 0
    assert result.output == "3.9.13  ReleaseLevel: final, Serial: 0\n"
